/*
 * (C) Copyright 2009
 * Texas Instruments, <www.ti.com>
 *
 * Author :
 *      Manikandan Pillai <mani.pillai@ti.com>
 *
 * Derived from Beagle Board and 3430 SDP code by
 *      Richard Woodruff <r-woodruff2@ti.com>
 *      Syed Mohammed Khasim <khasim@ti.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR /PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>
#include <asm/io.h>
#include <asm/arch/sys_proto.h>
#include <asm/arch/cpu.h>
#include <asm/arch/clock.h>
#include <asm/arch/ddr_defs.h>

#define OPP_CONFIG		100

extern omap3_sysinfo sysinfo;

/******************************************
 * get_cpu_rev(void) - extract rev info
 ******************************************/
u32 get_cpu_rev(void)
{
	u32 id;
	u32 rev;

	id = readl(DEVICE_ID);
	rev = (id >> 28) & 0xF;

#ifdef CONFIG_TI814X
	/* PG2.1 devices should read 0x3 as chip rev
	 * Some PG2.1 devices have 0xc as chip rev
	 */
	if (0x3 == rev || 0xc == rev)
		return PG2_1;
	else
		return PG1_0;
#endif
	return rev;
}

/******************************************
 * get_cpu_type(void) - extract cpu info
 ******************************************/
u32 get_cpu_type(void)
{
	u32 id = 0;
	u32 partnum;

	id = readl(DEVICE_ID);
	partnum = (id >> 12) & 0xffff;

	return partnum;
}

/*************************************************************************
 * get_board_rev() - setup to pass kernel board revision information
 * returns:(bit[0-3] sub version, higher bit[7-4] is higher version)
 *************************************************************************/
u32 get_board_rev(void)
{
	return 0x0;
}

/*************************************************************
 *  get_device_type(): tell if GP/HS/EMU/TST
 *************************************************************/
u32 get_device_type(void)
{
	int mode;
	mode = __raw_readl(CONTROL_STATUS) & (DEVICE_MASK);
	return(mode >>= 8);
}

/************************************************
 * get_sysboot_value(void) - return SYS_BOOT[4:0]
 ************************************************/
u32 get_sysboot_value(void)
{
	int mode;
	mode = __raw_readl(CONTROL_STATUS) & (SYSBOOT_MASK);
	return mode;
}

/******************************************
 * pg_val_ti816x() - runtime PG ver detect
 ******************************************/
u32 pg_val_ti816x(u32 pg1_val, u32 pg2_val)
{
	/* TI816X PG1.0 devices should read 0x0 as chip rev
	 * TI816X PG1.1 devices should read 0x1 as chip rev
	 */
	if (0x1 == get_cpu_rev())
		return pg2_val;
	else
		return pg1_val;
}

/***************************************************
 * u32 pg_val_ti814x() - return the PG specifi value
 ***************************************************/
u32 pg_val_ti814x(u32 pg1_val, u32 pg2_val)
{
	/* PG2.1 devices should read 0x3 as chip rev */
	if (PG2_1 == get_cpu_rev())
		return pg2_val;
	else
		return pg1_val;
}

/************************************************************
 * get_sysboot_bw(void) - return buswidth from CONTROL_STATUS
 ************************************************************/
u32 get_sysboot_bw(void)
{
	int bw;
	bw = __raw_readl(CONTROL_STATUS) & (SYSBOOT_BW_MASK);
	bw >>= SYSBOOT_BW_POS;
	if (bw == 0)	/* 8-bit nand if BTMODE BW pin on board is ON */
		return 0;
	else if (bw == 1)/* 16-bit nand if BTMODE BW pin on board is OFF */
		return 1;

	/* Failed to detect */
	return -1;
}

/***************************************************
 * u32 opp_val_dm385() - return the opp specifi value
 ***************************************************/
u32 opp_val_dm385(u32 opp100_val, u32 opp120_val)
{
	/* Get OPP usage */
	if (OPP_CONFIG == 100)
		return opp100_val;
	else
		return opp120_val;
}

#ifdef CONFIG_DISPLAY_CPUINFO
#define PRINT_ALL	1

void print_ddr3_sw_level_values (int emif)
{
#ifdef CONFIG_TI816X
    #ifdef CONFIG_TI816X_DDR3_SW_LEVELING

	    unsigned int emif0_rd_dqs_gate_L0,emif0_rd_dqs_gate_L1,emif0_rd_dqs_gate_L2,emif0_rd_dqs_gate_L3;	
	    unsigned int emif0_wr_dqs_L0,emif0_wr_dqs_L1,emif0_wr_dqs_L2,emif0_wr_dqs_L3;	
		unsigned int emif0_wr_data_L0,emif0_wr_data_L1,emif0_wr_data_L2,emif0_wr_data_L3;	
		unsigned int emif0_rd_dqs_L0,emif0_rd_dqs_L1,emif0_rd_dqs_L2,emif0_rd_dqs_L3;
		unsigned int emif1_rd_dqs_gate_L0,emif1_rd_dqs_gate_L1,emif1_rd_dqs_gate_L2,emif1_rd_dqs_gate_L3;
		unsigned int emif1_wr_dqs_L0,emif1_wr_dqs_L1,emif1_wr_dqs_L2,emif1_wr_dqs_L3;	
		unsigned int emif1_wr_data_L0,emif1_wr_data_L1,emif1_wr_data_L2,emif1_wr_data_L3;	
		unsigned int emif1_rd_dqs_L0,emif1_rd_dqs_L1,emif1_rd_dqs_L2,emif1_rd_dqs_L3;


		emif0_rd_dqs_gate_L0 = __raw_readl(DDRPHY_CONFIG_BASE + 0x108);	
		emif0_rd_dqs_gate_L1 = __raw_readl(DDRPHY_CONFIG_BASE + 0x1AC);
		emif0_rd_dqs_gate_L2 = __raw_readl(DDRPHY_CONFIG_BASE + 0x250);
		emif0_rd_dqs_gate_L3 = __raw_readl(DDRPHY_CONFIG_BASE + 0x2F4);

		emif0_wr_dqs_L0 = __raw_readl(DDRPHY_CONFIG_BASE + 0x0DC);	
		emif0_wr_dqs_L1 = __raw_readl(DDRPHY_CONFIG_BASE + 0x180);
		emif0_wr_dqs_L2 = __raw_readl(DDRPHY_CONFIG_BASE + 0x224);
		emif0_wr_dqs_L3 = __raw_readl(DDRPHY_CONFIG_BASE + 0x2C8);	

		emif0_wr_data_L0 = __raw_readl(DDRPHY_CONFIG_BASE + 0x120);	
		emif0_wr_data_L1 = __raw_readl(DDRPHY_CONFIG_BASE + 0x1C4);
		emif0_wr_data_L2 = __raw_readl(DDRPHY_CONFIG_BASE + 0x268);
		emif0_wr_data_L3 = __raw_readl(DDRPHY_CONFIG_BASE + 0x30C);		

		emif0_rd_dqs_L0 = __raw_readl(DDRPHY_CONFIG_BASE + 0x0C8);	
		emif0_rd_dqs_L1 = __raw_readl(DDRPHY_CONFIG_BASE + 0x16C);
		emif0_rd_dqs_L2 = __raw_readl(DDRPHY_CONFIG_BASE + 0x210);
		emif0_rd_dqs_L3 = __raw_readl(DDRPHY_CONFIG_BASE + 0x2B4);		

		emif1_rd_dqs_gate_L0 = __raw_readl(DDRPHY_CONFIG_BASE + 0x108);	
		emif1_rd_dqs_gate_L1 = __raw_readl(DDRPHY_CONFIG_BASE + 0x1AC);
		emif1_rd_dqs_gate_L2 = __raw_readl(DDRPHY_CONFIG_BASE + 0x250);
		emif1_rd_dqs_gate_L3 = __raw_readl(DDRPHY_CONFIG_BASE + 0x2F4);

		emif1_wr_dqs_L0 = __raw_readl(DDRPHY_CONFIG_BASE + 0x0DC);	
		emif1_wr_dqs_L1 = __raw_readl(DDRPHY_CONFIG_BASE + 0x180);
		emif1_wr_dqs_L2 = __raw_readl(DDRPHY_CONFIG_BASE + 0x224);
		emif1_wr_dqs_L3 = __raw_readl(DDRPHY_CONFIG_BASE + 0x2C8);	

		emif1_wr_data_L0 = __raw_readl(DDRPHY_CONFIG_BASE + 0x120);	
		emif1_wr_data_L1 = __raw_readl(DDRPHY_CONFIG_BASE + 0x1C4);
		emif1_wr_data_L2 = __raw_readl(DDRPHY_CONFIG_BASE + 0x268);
		emif1_wr_data_L3 = __raw_readl(DDRPHY_CONFIG_BASE + 0x30C);		

		emif1_rd_dqs_L0 = __raw_readl(DDRPHY_CONFIG_BASE + 0x0C8);	
		emif1_rd_dqs_L1 = __raw_readl(DDRPHY_CONFIG_BASE + 0x16C);
		emif1_rd_dqs_L2 = __raw_readl(DDRPHY_CONFIG_BASE + 0x210);
		emif1_rd_dqs_L3 = __raw_readl(DDRPHY_CONFIG_BASE + 0x2B4);

		if ( emif == 0 )
		{
	        printf ("----------SW LEVEL Info (EMIF %d) -------\n", emif);  
	        printf("RD_DQS_GATE_BYTE_LANE0: 0x%08X\n", emif0_rd_dqs_gate_L0);
	        printf("RD_DQS_GATE_BYTE_LANE1: 0x%08X\n", emif0_rd_dqs_gate_L1);
	        printf("RD_DQS_GATE_BYTE_LANE2: 0x%08X\n", emif0_rd_dqs_gate_L2);
	        printf("RD_DQS_GATE_BYTE_LANE3: 0x%08X\n", emif0_rd_dqs_gate_L3);
	        printf("\n");

	        printf("WR_DQS_RATIO_BYTE_LANE0: 0x%08X\n", emif0_wr_dqs_L0);
	        printf("WR_DQS_RATIO_BYTE_LANE1: 0x%08X\n", emif0_wr_dqs_L1);
	        printf("WR_DQS_RATIO_BYTE_LANE2: 0x%08X\n", emif0_wr_dqs_L2);
	        printf("WR_DQS_RATIO_BYTE_LANE3: 0x%08X\n", emif0_wr_dqs_L3);
	        printf("\n");

	        printf("RD_DQS_RATIO_BYTE_LANE0: 0x%08X\n", emif0_rd_dqs_L0);
	        printf("RD_DQS_RATIO_BYTE_LANE1: 0x%08X\n", emif0_rd_dqs_L1);
	        printf("RD_DQS_RATIO_BYTE_LANE2: 0x%08X\n", emif0_rd_dqs_L2);
	        printf("RD_DQS_RATIO_BYTE_LANE3: 0x%08X\n", emif0_rd_dqs_L3);
	        printf("\n");

	        printf("WR_DATA_RATIO_BYTE_LANE0: 0x%08X\n", emif0_wr_data_L0);
	        printf("WR_DATA_RATIO_BYTE_LANE1: 0x%08X\n", emif0_wr_data_L1);
	        printf("WR_DATA_RATIO_BYTE_LANE2: 0x%08X\n", emif0_wr_data_L2);
	        printf("WR_DATA_RATIO_BYTE_LANE3: 0x%08X\n", emif0_wr_data_L3);
			
		}	
		else if ( emif == 1 )
		{
	        printf ("----------SW LEVEL Info (EMIF %d) -------\n", emif);  
	        printf("RD_DQS_GATE_BYTE_LANE0: 0x%08X\n", emif1_rd_dqs_gate_L0);
	        printf("RD_DQS_GATE_BYTE_LANE1: 0x%08X\n", emif1_rd_dqs_gate_L1);
	        printf("RD_DQS_GATE_BYTE_LANE2: 0x%08X\n", emif1_rd_dqs_gate_L2);
	        printf("RD_DQS_GATE_BYTE_LANE3: 0x%08X\n", emif1_rd_dqs_gate_L3);
	        printf("\n");

	        printf("WR_DQS_RATIO_BYTE_LANE0: 0x%08X\n", emif1_wr_dqs_L0);
	        printf("WR_DQS_RATIO_BYTE_LANE1: 0x%08X\n", emif1_wr_dqs_L1);
	        printf("WR_DQS_RATIO_BYTE_LANE2: 0x%08X\n", emif1_wr_dqs_L2);
	        printf("WR_DQS_RATIO_BYTE_LANE3: 0x%08X\n", emif1_wr_dqs_L3);
	        printf("\n");

	        printf("RD_DQS_RATIO_BYTE_LANE0: 0x%08X\n", emif1_rd_dqs_L0);
	        printf("RD_DQS_RATIO_BYTE_LANE1: 0x%08X\n", emif1_rd_dqs_L1);
	        printf("RD_DQS_RATIO_BYTE_LANE2: 0x%08X\n", emif1_rd_dqs_L2);
	        printf("RD_DQS_RATIO_BYTE_LANE3: 0x%08X\n", emif1_rd_dqs_L3);
	        printf("\n");

	        printf("WR_DATA_RATIO_BYTE_LANE0: 0x%08X\n", emif1_wr_data_L0);
	        printf("WR_DATA_RATIO_BYTE_LANE1: 0x%08X\n", emif1_wr_data_L1);
	        printf("WR_DATA_RATIO_BYTE_LANE2: 0x%08X\n", emif1_wr_data_L2);
	        printf("WR_DATA_RATIO_BYTE_LANE3: 0x%08X\n", emif1_wr_data_L3);
		
		}
		
    #else
        printf ("------------ SW LEVELING Not Enabled------------\n");
    #endif
#else
        printf ("----------SW LEVEL Info (EMIF %d) -------\n", emif);  
        printf("DDR3_PHY_RD_DQS_CS0_BYTE0: 0x%08X\n", DDR3_PHY_RD_DQS_CS0_BYTE0);
        printf("DDR3_PHY_RD_DQS_CS0_BYTE1: 0x%08X\n", DDR3_PHY_RD_DQS_CS0_BYTE1);
        printf("DDR3_PHY_RD_DQS_CS0_BYTE2: 0x%08X\n", DDR3_PHY_RD_DQS_CS0_BYTE2);
        printf("DDR3_PHY_RD_DQS_CS0_BYTE3: 0x%08X\n", DDR3_PHY_RD_DQS_CS0_BYTE3);
        printf("\n");

        printf("DDR3_PHY_WR_DQS_CS0_BYTE0: 0x%08X\n", DDR3_PHY_WR_DQS_CS0_BYTE0);
        printf("DDR3_PHY_WR_DQS_CS0_BYTE1: 0x%08X\n", DDR3_PHY_WR_DQS_CS0_BYTE1);
        printf("DDR3_PHY_WR_DQS_CS0_BYTE2: 0x%08X\n", DDR3_PHY_WR_DQS_CS0_BYTE2);
        printf("DDR3_PHY_WR_DQS_CS0_BYTE3: 0x%08X\n", DDR3_PHY_WR_DQS_CS0_BYTE3);
        printf("\n");

        printf("DDR3_PHY_RD_DQS_GATE_CS0_BYTE0: 0x%08X\n", DDR3_PHY_RD_DQS_GATE_CS0_BYTE0);
        printf("DDR3_PHY_RD_DQS_GATE_CS0_BYTE1: 0x%08X\n", DDR3_PHY_RD_DQS_GATE_CS0_BYTE1);
        printf("DDR3_PHY_RD_DQS_GATE_CS0_BYTE2: 0x%08X\n", DDR3_PHY_RD_DQS_GATE_CS0_BYTE2);
        printf("DDR3_PHY_RD_DQS_GATE_CS0_BYTE3: 0x%08X\n", DDR3_PHY_RD_DQS_GATE_CS0_BYTE3);
        printf("\n");

        printf("DDR3_PHY_WR_DATA_CS0_BYTE0: 0x%08X\n", DDR3_PHY_WR_DATA_CS0_BYTE0);
        printf("DDR3_PHY_WR_DATA_CS0_BYTE1: 0x%08X\n", DDR3_PHY_WR_DATA_CS0_BYTE1);
        printf("DDR3_PHY_WR_DATA_CS0_BYTE2: 0x%08X\n", DDR3_PHY_WR_DATA_CS0_BYTE2);
        printf("DDR3_PHY_WR_DATA_CS0_BYTE3: 0x%08X\n", DDR3_PHY_WR_DATA_CS0_BYTE3);
#endif
    printf("\n");
}
/**
 * Print CPU information
 */
int print_cpuinfo (void)
{
	char *cpu_s, *sec_s;
	int arm_freq, ddr_freq , rev;
	#if PRINT_ALL
	int m3_freq, dss_freq , hdvicp_freq, l3fast_freq;
	#endif
	#ifdef CONFIG_DM385
	int l3_freq, dss_freq, iva_freq, iss_freq;
	#endif

	rev = get_cpu_rev();
	switch (get_cpu_type()) {
	case TI8168:
		cpu_s = "TI8168";
		break;
	case TI8148:
		cpu_s = "TI8148";
		break;
	case DM385:
		cpu_s = "DM385";
		break;
	default:
		cpu_s = "Unknown cpu type";
		break;
	}

	switch (get_device_type()) {
	case TST_DEVICE:
		sec_s = "TST";
		break;
	case EMU_DEVICE:
		sec_s = "EMU";
		break;
	case HS_DEVICE:
		sec_s = "HS";
		break;
	case GP_DEVICE:
		sec_s = "GP";
		break;
	default:
		sec_s = "?";
	}

	if (rev < PG_END) {
		char cpu_rev_str[5][4] = {"1.0", "1.1", "2.0", "2.1"}, *cpu_rev;

		cpu_rev = cpu_rev_str[rev];
		printf("%s-%s rev %s\n",
			cpu_s, sec_s, cpu_rev);
	} else {
		printf("%s-%s rev ?????[%1x]\n",
			cpu_s, sec_s, rev);
	}
	printf("\n");

	/* ARM and DDR frequencies */

#ifdef CONFIG_TI816X
	/* f0 = ((N * K) / (FREQ * P * M)) * fr */

	arm_freq = (((MAIN_N * FAPLL_K * OSC_FREQ)/
			( MAIN_P * MAIN_MDIV2 )))/SYSCLK_2_DIV;

	/*
	 * If the fractional part (MAIN_FRACFREQ2i) in non zero then the formula
	 * is : fo = ((N * K) / ( (INTFREQ + FRACFREQ) * p * M)) * fr
	 *
	 * For 13.824, INTFREQ = 0xD and FRACFREQ = 0xD2F1A9
	 * We are supposed to divide FRACFREQ by 0x1000000
	 *
	 * Due to overflow of values on multiplying by 0x1000000
	 * we discard the least 8 bits
	 */
	arm_freq = (arm_freq * (0x1000000 >> 8));
	arm_freq=arm_freq/((MAIN_INTFREQ2 * 0x1000000 + MAIN_FRACFREQ2)>>8);
	#if PRINT_ALL
	hdvicp_freq = (((MAIN_N * FAPLL_K * OSC_FREQ)/
			( MAIN_P * MAIN_MDIV3 )))/SYSCLK_2_DIV;

	hdvicp_freq = (hdvicp_freq * (0x1000000 >> 8));
	hdvicp_freq=hdvicp_freq/((MAIN_INTFREQ3 * 0x1000000 + MAIN_FRACFREQ3)>>8);

	l3fast_freq = (((MAIN_N * FAPLL_K * OSC_FREQ)/
			( MAIN_P * MAIN_MDIV4 )))/SYSCLK_2_DIV;

	l3fast_freq = (l3fast_freq * (0x1000000 >> 8));
	l3fast_freq=l3fast_freq/((MAIN_INTFREQ4 * 0x1000000 + MAIN_FRACFREQ4)>>8);

	dss_freq = l3fast_freq/2;
	m3_freq = l3fast_freq/2;
	#endif

	ddr_freq = ((DDR_N * OSC_FREQ)/DDR_MDIV1);
#else
	/* clk_out  = ((OSC_0/ ( N+1 )) * M) / M2   */
	arm_freq = ((OSC_0_FREQ / (MODENA_N + 1) * MODENA_M) / MODENA_M2);
	ddr_freq = ((OSC_0_FREQ / (DDR_N + 1) * DDR_M) / DDR_M2);
#ifdef CONFIG_DM385
	l3_freq = ((OSC_0_FREQ / (L3_N + 1) * L3_M) / L3_M2);
	dss_freq = ((OSC_0_FREQ / (DSS_N + 1) * DSS_M) / DSS_M2);
	iva_freq = ((OSC_0_FREQ / (IVA_N + 1) * IVA_M) / IVA_M2);
	iss_freq = ((OSC_0_FREQ / (ISS_N + 1) * ISS_M) / ISS_M2);
#endif
#endif
	printf("ARM clk: %dMHz\n", arm_freq);
	printf("DDR clk: %dMHz\n", ddr_freq);
	#if PRINT_ALL
	printf("HDVICP clk: %dMHz\n", hdvicp_freq);
	printf("L3 Fast clk: %dMHz\n", l3fast_freq);
	printf("HDVPSS clk: %dMHz\n", dss_freq);
	printf("Ducati M3 clk: %dMHz\n", m3_freq);
	#endif
    //print_ddr3_sw_level_values(0); /* EMIF 0 */
    //print_ddr3_sw_level_values(1); /* EMIF 1 */
#ifdef CONFIG_DM385
	printf("L3 clk: %dMHz\n", l3_freq);
	printf("DSS clk: %dMHz\n", dss_freq);
	printf("IVA clk: %dMHz\n", iva_freq);
	printf("ISS clk: %dMHz\n", iss_freq);
#endif
	printf("\n");

	return 0;
}
#endif	/* CONFIG_DISPLAY_CPUINFO */
