/*
 * Copyright (C) 2010 Texas Instruments
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * ----------------------------------------------------------------------------
 *
 */

#ifndef _DDR_DEFS_TI816X_H
#define _DDR_DEFS_TI816X_H

#include <asm/arch/hardware.h>

#ifdef CONFIG_TI816X_EVM_DDR3

#ifndef CONFIG_TI8168_DVR
#define CONFIG_TI816X_DDR3_796 /* Values supported 400,531,675,796 */
#endif

/* Enable software leveling as part of DDR3 init*/
#define CONFIG_TI816X_DDR3_SW_LEVELING


/*
 * DDR3 force values.  These are board dependent
 */

/* EVM 400 MHz clock Settings
 * EVM has only a single RANK (chip select) */
#define N_RANK			1

/*
 * Invert clock adds an additional half cycle delay on the command
 * interface.  The additional half cycle, is usually meant to enable
 * leveling in the situation that DQS is later than CK on the board.  It
 * also helps provide some additional margin for leveling.
 *
 * For the EVM this is helping us with additional room for the write
 * leveling.  Since the dqs delays are very small.
 */
#define INVERT_CLOCK		1

/*
 * CMD_SLAVE_RATIO determines where is the command placed with respect
 * to the clock edge.  This is a ratio, implying 0x100 is one cycle.
 * Ideally the command is centered so - this should be half cycle
 * delay (0x80).  But if invert clock is in use, an additional half
 * cycle must be added
 */
#define CMD_SLAVE_FROM_INV_CLOCK(i) (((i) == 0) ? 0x80 : 0x100)
#define CMD_SLAVE_RATIO		CMD_SLAVE_FROM_INV_CLOCK(INVERT_CLOCK)

#ifdef TI816X_DDR3_PG_1_0
/*
 * EMIF PHY allows for controlling write DQS delay w.r.t. clock.  The
 * value may be forced or use values determined from the leveling
 * process.  Since we are doing the leveling - these are actually
 * don't care and are not used.  The force is in delay units
 */
#define WR_DQS_FORCE_3	0x00000010
#define WR_DQS_FORCE_2	0x00000010
#define WR_DQS_FORCE_1	0x00000010
#define WR_DQS_FORCE_0	0x00000010

/*
 * EMIF PHY allows for controlling how much the read DQS must be
 * delayed to sample the data correctly.  Ideally this is about half a
 * cycle.  The force here is delay units.  The value here is in use -
 * as the current leveling process may not obtain this reliably.  The
 * value has been determined via various tests on the EVM and changing
 * this setting is no recomended.
 */
#define RD_DQS_FORCE_0		0x00000028
#define RD_DQS_FORCE_1		0x00000028
#define RD_DQS_FORCE_2		0x00000028
#define RD_DQS_FORCE_3		0x00000028

/*
 * read gate represents the round trip delay from command to read data
 * on the board + some package delay.  This is the period for which
 * the bust may be tristated between a write and a read command and
 * hence must not be sampled (gated off).  This is actually determined
 * via the read leveling process and hence this value is a don't care
 * for the EVM
 */
#define RD_GATE_FORCE_3	0x44
#define RD_GATE_FORCE_2	0x44
#define RD_GATE_FORCE_1	0x44
#define RD_GATE_FORCE_0	0x44

#endif

/*
 * This represents the initial value for the leveling process.  The
 * value is a ratio - so 0x100 represents one cycle.  The real delay
 * is determined through the leveling process.
 *
 * During the leveling process, 0x20 is subtracted from the value, so
 * we have added that to the value we want to set.  We also set the
 * values such that byte3 completes leveling after byte2 and byte1
 * after byte0.
 */
#define WR_DQS_RATIO_0		0x20
#define WR_DQS_RATIO_1		0x20
#define WR_DQS_RATIO_2		0x20
#define WR_DQS_RATIO_3		0x20

#ifdef TI816X_DDR3_PG_1_0
/*
 * read dqs ratio is only used in DDR2
 */
#define RD_DQS_RATIO_0		0x40
#define RD_DQS_RATIO_1		0x40
#define RD_DQS_RATIO_2		0x40
#define RD_DQS_RATIO_3		0x40
#endif

/*
 * This represents the initial value for the leveling process.  The
 * value is a ratio - so 0x100 represents one cycle.  The real delay
 * is determined through the leveling process.
 *
 * During the leveling process, 0x20 is subtracted from the value, so
 * we have added that to the value we want to set.  We also set the
 * values such that byte3 completes leveling after byte2 and byte1
 * after byte0.
 */
#define RD_GATE_RATIO_0	0x20
#define RD_GATE_RATIO_1	0x20
#define RD_GATE_RATIO_2	0x20
#define RD_GATE_RATIO_3	0x20

#ifdef TI816X_DDR3_PG_1_0
/*
 * currently there is an issue with the automatic training process for
 * DDR3 by setting the initial leveling ratios appropriately we are
 * able to work arround write leveling and read gate leveling.  How
 * ever the automatic process may not work well for the read eye
 * training (determining rd dqs delay).  To work arround this - we
 * leverage the pre-knowledge of a working RD DQS delay and make the
 * leveling process complete by forcing good and bad values
 * This is enabled via HACK_EYE_TRAINING
 */
#define HACK_EYE_TRAINING	0

/*
 * only the rd dqs delay needs to be forced.  Others are determined via the leveling process
 */
#define USE_WR_DQS_FORCE	0
#define USE_RD_DQS_FORCE	HACK_EYE_TRAINING
#define USE_RD_GATE_FORCE	0

#endif
/*
 * data rate in MHz.  The DDR clock will be 1/2 of this value
 */
#define DDR_DATA_RATE		800

#define USE_EMIF0		1
#define USE_EMIF1		1

/*
 * EMIF Paramters.  Refer the EMIF register documentation and the
 * memory datasheet for details
 */
/* For 400 MHz */
#if defined(CONFIG_TI816X_DDR3_400)
#define EMIF_TIM1    0x0CCCE524
#define EMIF_TIM2    0x30308023
#define EMIF_TIM3    0x009F82CF
#define EMIF_SDREF   0x10000C30
#define EMIF_SDCFG   0x62A41032
#define EMIF_PHYCFG  0x0000010B

#if defined(CONFIG_TI816X_DDR3_SW_LEVELING)
/* These values are obtained from the CCS app */
#define RD_DQS_GATE	((emif == 0) ? 0x12A : 0x12A)
#define RD_DQS		((emif == 0) ? 0x3B : 0x3B)
#define WR_DQS		((emif == 0) ? 0xA6 : 0xA6)
#endif

#endif	/* CONFIG_TI816X_DDR3_400 */

/* For 531 MHz */
#if defined(CONFIG_TI816X_DDR3_531)
#define EMIF_TIM1    0x0EF136AC
#define EMIF_TIM2    0x30408063
#define EMIF_TIM3    0x009F83AF
#define EMIF_SDREF   0x1000102E
#define EMIF_SDCFG   0x62A51832
#define EMIF_PHYCFG  0x0000010C

#if defined(CONFIG_TI816X_DDR3_SW_LEVELING)
/* These values are obtained from the CCS app */
#define RD_DQS_GATE	((emif == 0) ? 0x13D : 0x13D)
#define RD_DQS		((emif == 0) ? 0x39 : 0x39)
#define WR_DQS		((emif == 0) ? 0xB4 : 0xB4)
#endif

#endif /* CONFIG_TI816X_DDR_531 */

/* For 675 MHz */
#if defined(CONFIG_TI816X_DDR3_675)
/*
#define EMIF_TIM1    0x13358875
#define EMIF_TIM2    0x5051806C
#define EMIF_TIM3    0x009F84AF
#define EMIF_SDREF   0x10001491
#define EMIF_SDCFG   0x62A63032
#define EMIF_PHYCFG  0x0000010F
*/
//For 4G SAMSUNG K4B4G1646B-HCK0
#define EMIF_TIM1    0x13359825
#define EMIF_TIM2    0x40727FEB
#define EMIF_TIM3    0x00BF8AF8
#define EMIF_SDREF   0x10001491
#define EMIF_SDCFG   0x62A63032
#define EMIF_PHYCFG  0x0000010F

#if defined(CONFIG_TI816X_DDR3_SW_LEVELING)
/* These values are obtained from the CCS app */
#define RD_DQS_GATE	((emif == 0) ? 0x196 : 0x196)
#define RD_DQS		((emif == 0) ? 0x39 : 0x39)
#define WR_DQS		((emif == 0) ? 0x91 : 0x91)

#endif

#endif /* CONFIG_TI816X_DDR3_675 */

/* For 796 MHz */
#if defined(CONFIG_TI816X_DDR3_796)
//For 4G SAMSUNG K4B4G1646B-HCK0
//fine tune, 2012/11/02 Andrew//
#define EMIF_TIM1   0x1557B9AD
#define EMIF_TIM2   0x40D67FEB
#define EMIF_TIM3   0x00BF8CE8
#define EMIF_SDREF  0x10001840
#define EMIF_SDCFG  0x62C73B32
#define EMIF_PHYCFG 0x00100110
/*
#define EMIF_TIM1   0x1557B9AD
#define EMIF_TIM2   0x40C77FEB
#define EMIF_TIM3   0x001F8BFF
#define EMIF_SDREF  0x10001844
#define EMIF_SDCFG  0x62A73832
#define EMIF_PHYCFG 0x00100110
*/
/*
#define EMIF_TIM1   0x1779C9FE
#define EMIF_TIM2   0x50608074
#define EMIF_TIM3   0x009F857F
#define EMIF_SDREF  0x10001841
#define EMIF_SDCFG  0x62A73832
#define EMIF_PHYCFG 0x00000110
*/
#if defined(CONFIG_TI816X_DDR3_SW_LEVELING)
/* These values are obtained from the CCS app */
/*
#define RD_DQS_GATE    ((emif == 0) ? 0x1B3 : 0x1B3)
#define RD_DQS         ((emif == 0) ? 0x35 : 0x35)
#define WR_DQS         ((emif == 0) ? 0x93 : 0x93)
*/
#ifdef CONFIG_TI816X_VCS_DOLPHIN
//Dolphin A101
//	#define RD_DQS_GATE	((emif == 0) ? 0x134 : 0x13E)
//	#define RD_DQS		((emif == 0) ? 0x40 : 0x40)
//	#define WR_DQS		((emif == 0) ? 0xA0 : 0xA3)
    #define RD_DQS_GATE_LANE3       ((emif == 0) ? 0x159 : 0x130) /*BYTE3 OPT values from CCS slave ratio*/
    #define RD_DQS_GATE_LANE2       ((emif == 0) ? 0x152 : 0x126) /*BYTE2 OPT values from CCS slave ratio*/
    #define RD_DQS_GATE_LANE1       ((emif == 0) ? 0x13a : 0x15b) /*BYTE1 OPT values from CCS slave ratio*/
    #define RD_DQS_GATE_LANE0       ((emif == 0) ? 0x130 : 0x14f) /*BYTE0 OPT values from CCS slave ratio*/
    
    #define RD_DQS_LANE3            ((emif == 0) ? 0x3b : 0x3d) /*BYTE3 OPT values from CCS slave ratio*/
    #define RD_DQS_LANE2            ((emif == 0) ? 0x3b : 0x3a) /*BYTE2 OPT values from CCS slave ratio*/
    #define RD_DQS_LANE1            ((emif == 0) ? 0x3e : 0x3d) /*BYTE1 OPT values from CCS slave ratio*/
    #define RD_DQS_LANE0            ((emif == 0) ? 0x3b : 0x38) /*BYTE0 OPT values from CCS slave ratio*/
    
    #define WR_DQS_LANE3            ((emif == 0) ? 0xc7 : 0x9d) /*BYTE3 OPT values from CCS slave ratio*/
    #define WR_DQS_LANE2            ((emif == 0) ? 0xc3 : 0x99) /*BYTE2 OPT values from CCS slave ratio*/
    #define WR_DQS_LANE1            ((emif == 0) ? 0x9f : 0xb5) /*BYTE1 OPT values from CCS slave ratio*/
    #define WR_DQS_LANE0            ((emif == 0) ? 0x9d : 0xB3) /*BYTE0 OPT values from CCS slave ratio*/
    
    #define WR_DATA_LANE3           (WR_DQS_LANE3 + 0x40)
    #define WR_DATA_LANE2           (WR_DQS_LANE2 + 0x40)
    #define WR_DATA_LANE1           (WR_DQS_LANE1 + 0x40)
    #define WR_DATA_LANE0           (WR_DQS_LANE0 + 0x40)
#else
//Sonita A102-2 value, 2012/11/02 Andrew//
#define RD_DQS_GATE	((emif == 0) ? 0x174 : 0x16E)
#define RD_DQS		((emif == 0) ? 0x40 : 0x40)
#define WR_DQS		((emif == 0) ? 0xAA : 0xA6)
#endif //#ifdef CONFIG_TI816X_VCS_DOLPHIN

#endif

#endif /* CONFIG_TI816X_DDR_796 */


#if defined(CONFIG_TI816X_DDR3_SW_LEVELING)
#ifdef CONFIG_TI816X_VCS_DOLPHIN
//VCS
#define WR_DQS_RATIO_BYTE_LANE0		WR_DQS_LANE3
#define WR_DQS_RATIO_BYTE_LANE1		WR_DQS_LANE2
#define WR_DQS_RATIO_BYTE_LANE2		WR_DQS_LANE1
#define WR_DQS_RATIO_BYTE_LANE3		WR_DQS_LANE0

#define WR_DATA_RATIO_BYTE_LANE0	WR_DATA_LANE3
#define WR_DATA_RATIO_BYTE_LANE1	WR_DATA_LANE2
#define WR_DATA_RATIO_BYTE_LANE2	WR_DATA_LANE1
#define WR_DATA_RATIO_BYTE_LANE3	WR_DATA_LANE0

#define RD_DQS_RATIO_BYTE_LANE0		RD_DQS_LANE3
#define RD_DQS_RATIO_BYTE_LANE1		RD_DQS_LANE2
#define RD_DQS_RATIO_BYTE_LANE2		RD_DQS_LANE1
#define RD_DQS_RATIO_BYTE_LANE3		RD_DQS_LANE0

#define DQS_GATE_BYTE_LANE0		RD_DQS_GATE_LANE3
#define DQS_GATE_BYTE_LANE1		RD_DQS_GATE_LANE2
#define DQS_GATE_BYTE_LANE2		RD_DQS_GATE_LANE1
#define DQS_GATE_BYTE_LANE3		RD_DQS_GATE_LANE0
#else // CONFIG_TI816X_VCS_DOLPHIN
#define WR_DQS_RATIO_BYTE_LANE3	((WR_DQS << 10) | WR_DQS)
#define WR_DQS_RATIO_BYTE_LANE2	((WR_DQS << 10) | WR_DQS)
#define WR_DQS_RATIO_BYTE_LANE1	((WR_DQS << 10) | WR_DQS)
#define WR_DQS_RATIO_BYTE_LANE0	((WR_DQS << 10) | WR_DQS)

#define WR_DATA_RATIO_BYTE_LANE3	(((WR_DQS + 0x40) << 10) | (WR_DQS + 0x40))
#define WR_DATA_RATIO_BYTE_LANE2	(((WR_DQS + 0x40) << 10) | (WR_DQS + 0x40))
#define WR_DATA_RATIO_BYTE_LANE1	(((WR_DQS + 0x40) << 10) | (WR_DQS + 0x40))
#define WR_DATA_RATIO_BYTE_LANE0	(((WR_DQS + 0x40) << 10) | (WR_DQS + 0x40))

#define RD_DQS_RATIO			((RD_DQS << 10) | RD_DQS)

#define DQS_GATE_BYTE_LANE0		((RD_DQS_GATE << 10) | RD_DQS_GATE)
#define DQS_GATE_BYTE_LANE1		((RD_DQS_GATE << 10) | RD_DQS_GATE)
#define DQS_GATE_BYTE_LANE2		((RD_DQS_GATE << 10) | RD_DQS_GATE)
#define DQS_GATE_BYTE_LANE3		((RD_DQS_GATE << 10) | RD_DQS_GATE)
#endif // CONFIG_TI816X_VCS_DOLPHIN
#endif	/* CONFIG_TI816X_DDR3_SW_LEVELING */

#endif	/* CONFIG_TI816X_EVM_DDR3 */

#ifdef CONFIG_TI816X_EVM_DDR2

#define INVERT_CLK_OUT		0x0
#define CMD_SLAVE_RATIO		0x80
/*
 * DDR2 ratio values.  These are board dependent
 * obtained from sweep experiments
 */

/* EVM 400 MHz clock Settings */
#ifdef CONFIG_TI8168_DVR

#define WR_DQS_RATIO_BYTE_LANE3   ((0x40 << 10) | 0x40)
#define WR_DQS_RATIO_BYTE_LANE2   ((0x40 << 10) | 0x40)
#define WR_DQS_RATIO_BYTE_LANE1   ((0x40 << 10) | 0x40)
#define WR_DQS_RATIO_BYTE_LANE0   ((0x40 << 10) | 0x40)

#define WR_DATA_RATIO_BYTE_LANE3  (((0x40 + 0x40) << 10) | (0x40 + 0x40))
#define WR_DATA_RATIO_BYTE_LANE2  (((0x40 + 0x40) << 10) | (0x40 + 0x40))
#define WR_DATA_RATIO_BYTE_LANE1  (((0x40 + 0x40) << 10) | (0x40 + 0x40))
#define WR_DATA_RATIO_BYTE_LANE0  (((0x40 + 0x40) << 10) | (0x40 + 0x40))

#define RD_DQS_RATIO              ((0x40 << 10) | 0x40)

#define DQS_GATE_BYTE_LANE0       ((0x122 << 10) | 0x122)
#define DQS_GATE_BYTE_LANE1       ((0x122 << 10) | 0x122)
#define DQS_GATE_BYTE_LANE2       ((0x122 << 10) | 0x122)
#define DQS_GATE_BYTE_LANE3       ((0x122 << 10) | 0x122)

#else

#define WR_DQS_RATIO_BYTE_LANE3	((0x4a << 10) | 0x4a)
#define WR_DQS_RATIO_BYTE_LANE2	((0x4a << 10) | 0x4a)
#define WR_DQS_RATIO_BYTE_LANE1	((0x4a << 10) | 0x4a)
#define WR_DQS_RATIO_BYTE_LANE0	((0x4a << 10) | 0x4a)

#define WR_DATA_RATIO_BYTE_LANE3	(((0x4a + 0x40) << 10) | (0x4a + 0x40))
#define WR_DATA_RATIO_BYTE_LANE2	(((0x4a + 0x40) << 10) | (0x4a + 0x40))
#define WR_DATA_RATIO_BYTE_LANE1	(((0x4a + 0x40) << 10) | (0x4a + 0x40))
#define WR_DATA_RATIO_BYTE_LANE0	(((0x4a + 0x40) << 10) | (0x4a + 0x40))

#define RD_DQS_RATIO			((0x40 << 10) | 0x40)

#define DQS_GATE_BYTE_LANE0		((0x13a << 10) | 0x13a)
#define DQS_GATE_BYTE_LANE1		((0x13a << 10) | 0x13a)
#define DQS_GATE_BYTE_LANE2		((0x13a << 10) | 0x13a)
#define DQS_GATE_BYTE_LANE3		((0x13a << 10) | 0x13a)

#endif

/*
 * EMIF Paramters
 */
#define EMIF_TIM1    0xAAB15E2
#define EMIF_TIM2    0x423631D2
#define EMIF_TIM3    0x80032F
#define EMIF_SDREF   0x10000C30
#define EMIF_SDCFG   0x43801A3A  /* 32 bit ddr2, CL=6, CWL=5, 13 rows, 8 banks, 10 bit column, 2 CS */
/*
 * TI816x PG1.0 EMIF_PHYCFG 0x0000030B   local odt = 3, read latency = 11
 * TI816x PG1.1 EMIF_PHYCFG 0x0000010B   local odt = 1, read latency = 11
 */
#define EMIF_PHYCFG  (pg_val_ti816x(0x0000030B, 0x0000010B))

#endif	/* CONFIG_TI816X_EVM_DDR2 */

#ifdef CONFIG_TI8168_DVR
//# phoong --------------------------------------------------------------------
// DVR_SDRTIM1		0x088B1592
// DVR_SDRTIM2		0x403631D2
// DVR_SDRTIM3		0x00800328
// DVR_SDRRCR		0x10000C30
// DVR_SDRCR		0x438016B2
// DVR_DDRPHYCR		0x0000030B

/* DVR DDR2	K4T1G084QQ-HCE7 (DDR2-800 5-5-5, 1Gb, dvr_netra)
* tCK		2.5
* tRP		12.5
* tRCD		12.5
* tWR		15
* tRAS(min)	45
* tRC		57.5
* tRRD		7.5
* tWTR		7.5
*
* T_RP=(tRP/tCK)-1 = (12.5/2.5)-1 = 4 (5)
* T_RCD=(tRCD/tCK)-1 = (12.5/2.5)-1 = 4 (5)
* T_WR=(tWR/tCK)-1 = (15/2.5)-1 = 5
* T_RAS=(tRAS/tCK)-1 = (45/2.5)-1 = 17
* T_RC=(tRC/tCK)-1 = (57.5/2.5)-1 = 22
* T_RRD=(tRRD/tCK)-1 = (7.5/2.5)-1 = 2 (4)
* T_WTR=(tWTR/tCK)-1 = (7.5/2.5)-1 = 2
*/
#define DDR2_T_RP	4
#define DDR2_T_RCD	4
#define DDR2_T_WR	5
#define DDR2_T_RAS	17
#define DDR2_T_RC	22
#define DDR2_T_RRD	2
#define DDR2_T_WTR	2

#define DVR_SDRTIM1	((DDR2_T_RP<<25)|(DDR2_T_RCD<<21)|(DDR2_T_WR<<17)|(DDR2_T_RAS<<12)| \
					 (DDR2_T_RC<<6)|(DDR2_T_RRD<<3)|(DDR2_T_WTR<<0))

/*
* tXP		2
* tXSNR		tRFC+10=127.5(1Gb)+10 =137.5
* tXSRD		200
* tRTP		7.5
* tCKE		3
*
* T_XP=greater then tXP or tCKE -> 4
* T_XSNR=(tXSNR/tCK)-1 = (137.5/2.5)-1 = 54
* T_XSRD=tXSRD-1 = (200-1) = 199
* T_RTP=(tRTP/tCK)-1 = (7.5/2.5)-1 = 2
* T_CKE=(tCKE/tCK)-1 = (3/2.5)-1 = 0.2 -> 1 (2)
*/
#define DDR2_T_XP	4
#define DDR2_T_XSNR	54
#define DDR2_T_XSRD	199
#define DDR2_T_RTP	2
#define DDR2_T_CKE	1

#define DVR_SDRTIM2	((DDR2_T_XP<<28)|(DDR2_T_XSNR<<16)|(DDR2_T_XSRD<<6)|(DDR2_T_RTP<<3)|(DDR2_T_CKE<<0))

/*
* tCKESR	ddr3
* tZQCS		ddr3
* tRFC		127.5(1Gb)
* tRAS(max)	70000
* Refresh rate	7.8u
*
* T_CKESR=(tCKESR/tCK)-1
* T_ZQCS=tZQCS-1 (read only)
* T_RFC=(tRFC/tCK)-1 = (127.5/2.5)-1 = 50
* T_RASMAX=(tRAS(max)/Refreshrate)-1 = (70000/7800)-1 = 8 (16)
*/
#define DDR3_T_CKESR	4		//effect DDR3
#define DDR2_T_RFC		50
#define DDR2_T_RASMAX	8

#define DVR_SDRTIM3	((DDR3_T_CKESR<<21)|(DDR2_T_RFC<<4)|(DDR2_T_RASMAX<<0))

/*
* SDRAM Configuration Register
*/
#define DDR_MEMTYPE		2		//# 2:DDR2, 3:DDR3
#define DDR_IBANKPOS	0		//# 0:Refer Table
#define DDR_DDRTERM		3		//# (DDR2)1:75Ohm, 2:150Ohm, 3:50Ohm
#define DDR_DDQS		1		//# 0:Single Ended DQS, 1:Differential Ended DQS
#define DDR_DYNODT		0		//# (DDR3)0:OFF, 1:RZQ/4, 2:RZQ/2
#define DDR_DLL			0		//# 0:Enable DLL, 1:Disable DLL
#define DDR_DRIVE		0		//# (DDR2) 0:Full Drive, 1:Reduced Drive
#define DDR_CWL			0		//# (DDR3)0:5, 1:6, 2:7, 3:8
#define DDR_NM			0		//# 0:32bits, 1:16bits
#define DDR_CL			5		//# (DDR2)CAS Latency 2~7
#define DDR_RSIZE		5		//# 0:9 Rows bits, 5:14 Rows bits, ..., 7:16 Rows bits
#define DDR_IBANK		3		//# 0:1 Bank, 1:2 Banks, 2:4 Banks, 3: 8 Banks
#define DDR_EBNK		0		//# 0:CS0 only, 1:CS0,CS1
#define DDR_PAGESIZE	2		//# 0:256-8col, 1:512-9col, 2:1024-10col, 3:2048-11col

#define DVR_SDRCR	((DDR_MEMTYPE<<29)|(DDR_IBANKPOS<<27)|(DDR_DDRTERM<<24)|(DDR_DDQS<<23)| \
					 (DDR_DYNODT<<21)|(DDR_DLL<<20)|(DDR_DRIVE<<18)|(DDR_CWL<<16)|(DDR_NM<<14)| \
					 (DDR_CL<<10)|(DDR_RSIZE<<7)|(DDR_IBANK<<4)|(DDR_EBNK<<3)|(DDR_PAGESIZE<<0))

/*
* SDRAM Refresh Control Register
*/
#define DDR_INITREF_DIS	0		//# 0:Enable Initialization and Refreshes, 1:Disable
#define	DDR_SRT			0		//# 0:Normal Operating Temperature range, 1:Extended
#define DDR_ASR			1		//# 0:Manual Self Refresh, 1:Auto
#define DDR_PASR		0		//# 0:Full Array
#define DDR_RR			3120	//# Refresh rate, Clock freq(MHz)xRefresh rate(uS)=400*7.8= 3120(0xC30)

#define DVR_SDRRCR	((DDR_INITREF_DIS<<31)|(DDR_SRT<<29)|(DDR_ASR<<28)|(DDR_PASR<<24)| \
					 (DDR_RR<<0))

/*
* DDR PHY Control Register
*
* read latency	max:CL(5)+7=12, min:CL(5)+1=6
*/
#define DDR_WR_LODT		0
#define DDR_RD_LODT		3		//# 50-Ohm ??
#define DDR_RD_LATENCY	11		//# read latency(max?)-1

#define DVR_DDRPHYCR	((DDR_WR_LODT<<10)|(DDR_RD_LODT<<8)|(DDR_RD_LATENCY<<0))
#endif	//# CONFIG_TI8168_DVR

#endif  /* _DDR_DEFS_TI816X_H */

