/*
 * Copyright (C) 2009, Texas Instruments, Incorporated
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation version 2.
 *
 * This program is distributed "as is" WITHOUT ANY WARRANTY of any
 * kind, whether express or implied; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#ifndef __TI81XX_HARDWARE_H
#define __TI81XX_HARDWARE_H

/* The objective is to keep only the overall memory map here
 * The break-up of the memory map for individual modules registers should
 * in a diff file like cpu.h so that when this is the only place
 * where change is needed for new SoCs when the ip is the same
 */
#define SRAM0_START			0x40300000

#define UART0_BASE			0x48020000
#define UART1_BASE			0x48022000
#define UART2_BASE			0x48024000

/* McSPI register */
/* In ti814x the following address if for MCSPI0 */
#define OMAP3_MCSPI1_BASE		0x48030000
/* TODO Cross check and move to cpu.h or the config file */
#define OMAP3_MCSPI_MAX_FREQ		48000000

/* DM Timer base addresses */
#define DM_TIMER0_BASE			0x4802C000
#define DM_TIMER1_BASE			0x4802E000
#define DM_TIMER2_BASE			0x48040000
#define DM_TIMER3_BASE			0x48042000
#define DM_TIMER4_BASE			0x48044000
#define DM_TIMER5_BASE			0x48046000
#define DM_TIMER6_BASE			0x48048000
#define DM_TIMER7_BASE			0x4804A000

/* GPIO Base address */
#define GPIO0_BASE				0x48032000
#define GPIO1_BASE				0x4804C000

#define GPIO_OE					0x134
#define GPIO_DATAIN				0x138
#define GPIO_CLEARDATAOUT		0x190
#define GPIO_SETDATAOUT			0x194

/* BCH Error Location Module */
#define ELM_BASE			0x48080000

/* Watchdog Timer */
#ifdef CONFIG_TI814X
#define WDT_BASE			0x481C7000
#else
#define WDT_BASE			0x480C2000
#endif

/* Control Module Base Address */
#define CTRL_BASE			0x48140000

/* PRCM Base Address */
#define PRCM_BASE			0x48180000

/* PLL Subsystem Base Address */
#ifdef CONFIG_TI814X
#define PLL_SUBSYS_BASE			0x481C5000
#endif

/* EMIF Base address */
#define EMIF4_0_CFG_BASE		0x4C000000
#define EMIF4_1_CFG_BASE		0x4D000000
#define DMM_BASE			0x4E000000

#ifdef CONFIG_TI816X
#define DDRPHY_0_CONFIG_BASE		0x48198000
#define DDRPHY_1_CONFIG_BASE		0x4819a000
#define DDRPHY_CONFIG_BASE 		((emif == 0) ? DDRPHY_0_CONFIG_BASE:DDRPHY_1_CONFIG_BASE)
#endif

#ifdef CONFIG_TI814X
#define DDRPHY_0_CONFIG_BASE		0x47C0C400
#define DDRPHY_1_CONFIG_BASE		0x47C0C800
#define DDRPHY_CONFIG_BASE 		((emif == 0) ? DDRPHY_0_CONFIG_BASE:DDRPHY_1_CONFIG_BASE)
#endif


/* GPMC Base address */
#define GPMC_BASE			0x50000000

/* CPSW Config space */
#define TI814X_CPSW_BASE               0x4A100000
#define TI814X_CPSW_MDIO_BASE          0x4A100800

/* MUX Pin */
#ifdef CONFIG_TI8168_DVR
/*
 * PTD  - Pull type Down
 * PTU  - Pull type Up
 * DIS  - Pull type selection is inactive
 * EN   - Pull type selection is active
 * M0   - Mode 0
 */

#define  PTU	(1 << 4)
#define  PTD	(0 << 4)
#define  EN		(1 << 3)
#define  DIS	(0 << 3)

#define  M0	0
#define  M1	1
#define  M2	2
#define  M3	3

#define MUX_VAL(offset, value) \
		__raw_writel(value, (TI816X_CTRL_BASE+PINCtrl_##offset));

/*
 * To get the actual address the offset has to added
 * with TI816X_CTRL_BASE to get the actual address
 */

/* TODO: VB__These register names will change.
 * The default mux config is not used right now.
 * Retaining for future when U-Boot will need to do some mux setup
 */
#define TI816X_CTRL_BASE				CTRL_BASE

#define PINCtrl_TSI0_DCLK       0x0800
#define PINCtrl_TSI0_DATA       0x0804
#define PINCtrl_TSI0_BYTSTRT    0x0808
#define PINCtrl_TSI0_PACVAL     0x080C
#define PINCtrl_TSI0_PACERR     0x0810
#define PINCtrl_TSI1_DCLK       0x0814
#define PINCtrl_TSI1_DATA       0x0818
#define PINCtrl_TSI1_BYTSTRT    0x081C
#define PINCtrl_TSI1_PACVAL     0x0820
#define PINCtrl_TSI1_PACERR     0x0824
#define PINCtrl_TSI2_DCLK       0x0828
#define PINCtrl_TSI2_DATA       0x082C
#define PINCtrl_TSI2_BYTSTRT    0x0830
#define PINCtrl_TSI2_PACVAL     0x0834
#define PINCtrl_TSI2_PACERR     0x0838
#define PINCtrl_TSI3_DCLK       0x083C
#define PINCtrl_TSI3_DATA       0x0840
#define PINCtrl_TSI3_BYTSTRT    0x0844
#define PINCtrl_TSI3_PACVAL     0x0848
#define PINCtrl_TSI3_PACERR     0x084C
#define PINCtrl_TSI4_DCLK       0x0850
#define PINCtrl_TSI4_DATA       0x0854
#define PINCtrl_TSI4_BYTSTRT    0x0858
#define PINCtrl_TSI4_PACVAL     0x085C
#define PINCtrl_TSI4_PACERR     0x0860
#define PINCtrl_TSI5_DCLK       0x0864
#define PINCtrl_TSI5_DATA       0x0868
#define PINCtrl_TSI5_BYTSTRT    0x086C
#define PINCtrl_TSI5_PACVAL     0x0870
#define PINCtrl_TSI5_PACERR     0x0874
#define PINCtrl_TSI6_DCLK       0x0878
#define PINCtrl_TSI6_DATA       0x087C
#define PINCtrl_TSI6_BYTSTRT    0x0880
#define PINCtrl_TSI6_PACVAL     0x0884
#define PINCtrl_TSI6_PACERR     0x0888
#define PINCtrl_TSI7_DCLK       0x088C
#define PINCtrl_TSI7_DATA       0x0890
#define PINCtrl_TSI7_BYTSTRT    0x0894
#define PINCtrl_TSI7_PACVAL     0x0898
#define PINCtrl_TSI7_PACERR     0x089C
#define PINCtrl_TSO0_DCLK       0x08A0
#define PINCtrl_TSO0_DATA       0x08A4
#define PINCtrl_TSO0_BYTSTRT    0x08A8
#define PINCtrl_TSO0_PACVAL     0x08AC
#define PINCtrl_TSO0_PACERR     0x08B0
#define PINCtrl_TSO1_DCLK       0x08B4
#define PINCtrl_TSO1_DATA       0x08B8
#define PINCtrl_TSO1_BYTSTRT    0x08BC
#define PINCtrl_TSO1_PACVAL     0x08C0
#define PINCtrl_TSO1_PACERR     0x08C4
#define PINCtrl_MTSI_DCLK       0x08C8
#define PINCtrl_MTSI_DATA0      0x08CC
#define PINCtrl_MTSI_DATA1      0x08D0
#define PINCtrl_MTSI_DATA2      0x08D4
#define PINCtrl_MTSI_DATA3      0x08D8
#define PINCtrl_MTSI_DATA4      0x08DC
#define PINCtrl_MTSI_DATA5      0x08E0
#define PINCtrl_MTSI_DATA6      0x08E4
#define PINCtrl_MTSI_DATA7      0x08E8
#define PINCtrl_MTSI_BYTSTRT    0x08EC
#define PINCtrl_MTSO_DCLK       0x08F0
#define PINCtrl_MTSO_DATA0      0x08F4
#define PINCtrl_MTSO_DATA1      0x08F8
#define PINCtrl_MTSO_DATA2      0x08FC
#define PINCtrl_MTSO_DATA3      0x0900
#define PINCtrl_MTSO_DATA4      0x0904
#define PINCtrl_MTSO_DATA5      0x0908
#define PINCtrl_MTSO_DATA6      0x090C
#define PINCtrl_MTSO_DATA7      0x0910
#define PINCtrl_MTSO_BYTSTRT    0x0914
#define PINCtrl_MCTL_SCLK       0x0918
#define PINCtrl_MCTL_SCTL       0x091C
#define PINCtrl_MCTL_SDI        0x0920
#define PINCtrl_MCTL_SDO        0x0924
#define PINCtrl_MCARD_MDET      0x0928
#define PINCtrl_MCARD_CD1       0x092C
#define PINCtrl_MCARD_CD2       0x0930
#define PINCtrl_MCARD_VS1       0x0934
#define PINCtrl_MCARD_VS2       0x0938
#define PINCtrl_MCARD_VCCEN     0x093C
#define PINCtrl_MCARD_VPPEN     0x0940
#define PINCtrl_MCARD_RESET     0x0944
#define PINCtrl_VIN0_CLK0       0x0948
#define PINCtrl_VIN0_CLK1       0x094C
#define PINCtrl_VIN0_D0         0x0950
#define PINCtrl_VIN0_D1         0x0954
#define PINCtrl_VIN0_D2         0x0958
#define PINCtrl_VIN0_D3         0x095C
#define PINCtrl_VIN0_D4         0x0960
#define PINCtrl_VIN0_D5         0x0964
#define PINCtrl_VIN0_D6         0x0968
#define PINCtrl_VIN0_D7         0x096C
#define PINCtrl_VIN0_D8         0x0970
#define PINCtrl_VIN0_D9         0x0974
#define PINCtrl_VIN0_D10        0x0978
#define PINCtrl_VIN0_D11        0x097C
#define PINCtrl_VIN0_D12        0x0980
#define PINCtrl_VIN0_D13        0x0984
#define PINCtrl_VIN0_D14        0x0988
#define PINCtrl_VIN0_D15        0x098C
#define PINCtrl_VOUT0_CLK       0x0990
#define PINCtrl_VOUT0_G_Y_YC2   0x0994
#define PINCtrl_VOUT0_G_Y_YC3   0x0998
#define PINCtrl_VOUT0_G_Y_YC4   0x099C
#define PINCtrl_VOUT0_G_Y_YC5   0x09A0
#define PINCtrl_VOUT0_G_Y_YC6   0x09A4
#define PINCtrl_VOUT0_G_Y_YC7   0x09A8
#define PINCtrl_VOUT0_G_Y_YC8   0x09AC
#define PINCtrl_VOUT0_G_Y_YC9   0x09B0
#define PINCtrl_VOUT0_B_CB_C2   0x09B4
#define PINCtrl_VOUT0_B_CB_C3   0x09B8
#define PINCtrl_VOUT0_B_CB_C4   0x09BC
#define PINCtrl_VOUT0_B_CB_C5   0x09C0
#define PINCtrl_VOUT0_B_CB_C6   0x09C4
#define PINCtrl_VOUT0_B_CB_C7   0x09C8
#define PINCtrl_VOUT0_B_CB_C8   0x09CC
#define PINCtrl_VOUT0_B_CB_C9   0x09D0
#define PINCtrl_VOUT0_R_CR2     0x09D4
#define PINCtrl_VOUT0_R_CR3     0x09D8
#define PINCtrl_VOUT0_R_CR4     0x09DC
#define PINCtrl_VOUT0_R_CR5     0x09E0
#define PINCtrl_VOUT0_R_CR6     0x09E4
#define PINCtrl_VOUT0_R_CR7     0x09E8
#define PINCtrl_VOUT0_R_CR8     0x09EC
#define PINCtrl_VOUT0_R_CR9     0x09F0
#define PINCtrl_MCA0_ACLKR      0x09F4
#define PINCtrl_MCA0_AHCLKR     0x09F8
#define PINCtrl_MCA0_AFSR       0x09FC
#define PINCtrl_MCA0_ACLKX      0x0A00
#define PINCtrl_MCA0_AHCLKX     0x0A04
#define PINCtrl_MCA0_AFSX       0x0A08
#define PINCtrl_MCA0_AMUTE      0x0A0C
#define PINCtrl_MCA0_AXR0       0x0A10
#define PINCtrl_MCA0_AXR1       0x0A14
#define PINCtrl_MCA0_AXR2       0x0A18
#define PINCtrl_MCA0_AXR3       0x0A1C
#define PINCtrl_MCA0_AXR4       0x0A20
#define PINCtrl_MCA0_AXR5       0x0A24
#define PINCtrl_MCA1_ACLKR      0x0A28
#define PINCtrl_MCA1_AHCLKR     0x0A2C
#define PINCtrl_MCA1_AFSR       0x0A30
#define PINCtrl_MCA1_ACLKX      0x0A34
#define PINCtrl_MCA1_AHCLKX     0x0A38
#define PINCtrl_MCA1_AFSX       0x0A3C
#define PINCtrl_MCA1_AMUTE      0x0A40
#define PINCtrl_MCA1_AXR0       0x0A44
#define PINCtrl_MCA1_AXR1       0x0A48
#define PINCtrl_MCA2_ACLKR      0x0A4C
#define PINCtrl_MCA2_AHCLKR     0x0A50
#define PINCtrl_MCA2_AFSR       0x0A54
#define PINCtrl_MCA2_ACLKX      0x0A58
#define PINCtrl_MCA2_AHCLKX     0x0A5C
#define PINCtrl_MCA2_AFSX       0x0A60
#define PINCtrl_MCA2_AMUTE      0x0A64
#define PINCtrl_MCA2_AXR0       0x0A68
#define PINCtrl_MCA2_AXR1       0x0A6C
#define PINCtrl_MMC_POW         0x0A70
#define PINCtrl_MMC_CLK         0x0A74
#define PINCtrl_MMC_CMD         0x0A78
#define PINCtrl_MMC_DAT0        0x0A7C
#define PINCtrl_MMC_DAT1_SDIRQ  0x0A80
#define PINCtrl_MMC_DAT2_SDRW   0x0A84
#define PINCtrl_MMC_DAT3        0x0A88
#define PINCtrl_MMC_SDCD        0x0A8C
#define PINCtrl_MMC_SDWP        0x0A90
#define PINCtrl_SPI_SCLK        0x0A94
#define PINCtrl_SPI_SCS0        0x0A98
#define PINCtrl_SPI_SCS1        0x0A9C
#define PINCtrl_SPI_SCS2        0x0AA0
#define PINCtrl_SPI_SCS3        0x0AA4
#define PINCtrl_SPI_D0          0x0AA8
#define PINCtrl_SPI_D1          0x0AAC
#define PINCtrl_UART0_RXD       0x0AB0
#define PINCtrl_UART0_TXD       0x0AB4
#define PINCtrl_UART0_RTSN      0x0AB8
#define PINCtrl_UART0_CTSN      0x0ABC
#define PINCtrl_UART0_DTRN      0x0AC0
#define PINCtrl_UART0_DSRN      0x0AC4
#define PINCtrl_UART0_DCDN      0x0AC8
#define PINCtrl_UART0_RIN       0x0ACC
#define PINCtrl_UART1_RXD       0x0AD0
#define PINCtrl_UART1_TXD       0x0AD4
#define PINCtrl_UART1_RTSN      0x0AD8
#define PINCtrl_UART1_CTSN      0x0ADC
#define PINCtrl_UART2_RXD       0x0AE0
#define PINCtrl_UART2_TXD       0x0AE4
#define PINCtrl_UART2_RTSN      0x0AE8
#define PINCtrl_UART2_CTSN      0x0AEC
#define PINCtrl_SC0_DET         0x0AF0
#define PINCtrl_SC0_RST         0x0AF4
#define PINCtrl_SC0_CLK         0x0AF8
#define PINCtrl_SC0_DATA        0x0AFC
#define PINCtrl_SC0_C4          0x0B00
#define PINCtrl_SC0_VCCEN       0x0B04
#define PINCtrl_SC0_VPPEN       0x0B08
#define PINCtrl_SC1_DET         0x0B0C
#define PINCtrl_SC1_RST         0x0B10
#define PINCtrl_SC1_CLK         0x0B14
#define PINCtrl_SC1_DATA        0x0B18
#define PINCtrl_SC1_C4          0x0B1C
#define PINCtrl_SC1_VCCEN       0x0B20
#define PINCtrl_SC1_VPPEN       0x0B24
#define PINCtrl_TIM4_OUT        0x0B28
#define PINCtrl_TIM5_OUT        0x0B2C
#define PINCtrl_TIM6_OUT        0x0B30
#define PINCtrl_TIM7_OUT        0x0B34
#define PINCtrl_GPMC_CS0        0x0B38
#define PINCtrl_GPMC_CS1        0x0B3C
#define PINCtrl_GPMC_CS2        0x0B40
#define PINCtrl_GPMC_CS3        0x0B44
#define PINCtrl_GPMC_CS4        0x0B48
#define PINCtrl_GPMC_CS5        0x0B4C
#define PINCtrl_GPMC_WEN        0x0B50
#define PINCtrl_GPMC_OEN_REN    0x0B54
#define PINCtrl_GPMC_BE0N_CLE   0x0B58
#define PINCtrl_GPMC_BE1N       0x0B5C
#define PINCtrl_GPMC_ADVN_ALE   0x0B60
#define PINCtrl_GPMC_DIR        0x0B64
#define PINCtrl_GPMC_WPN        0x0B68
#define PINCtrl_GPMC_WAIT       0x0B6C
#define PINCtrl_GPMC_A0         0x0B70
#define PINCtrl_GPMC_A1         0x0B74
#define PINCtrl_GPMC_A2         0x0B78
#define PINCtrl_GPMC_A3         0x0B7C
#define PINCtrl_GPMC_A4         0x0B80
#define PINCtrl_GPMC_A5         0x0B84
#define PINCtrl_GPMC_A6         0x0B88
#define PINCtrl_GPMC_A7         0x0B8C
#define PINCtrl_GPMC_A8         0x0B90
#define PINCtrl_GPMC_A9         0x0B94
#define PINCtrl_GPMC_A10        0x0B98
#define PINCtrl_GPMC_A11        0x0B9C
#define PINCtrl_GPMC_A27        0x0BA0
#define PINCtrl_GPMC_D0         0x0BA4
#define PINCtrl_GPMC_D1         0x0BA8
#define PINCtrl_GPMC_D2         0x0BAC
#define PINCtrl_GPMC_D3         0x0BB0
#define PINCtrl_GPMC_D4         0x0BB4
#define PINCtrl_GPMC_D5         0x0BB8
#define PINCtrl_GPMC_D6         0x0BBC
#define PINCtrl_GPMC_D7         0x0BC0
#define PINCtrl_GPMC_D8         0x0BC4
#define PINCtrl_GPMC_D9         0x0BC8
#define PINCtrl_GPMC_D10        0x0BCC
#define PINCtrl_GPMC_D11        0x0BD0
#define PINCtrl_GPMC_D12        0x0BD4
#define PINCtrl_GPMC_D13        0x0BD8
#define PINCtrl_GPMC_D14        0x0BDC
#define PINCtrl_GPMC_D15        0x0BE0
#define PINCtrl_GPMC_CLK        0x0BE4
#define PINCtrl_GMII0_COL       0x0BE8
#define PINCtrl_GMII0_CRS       0x0BEC
#define PINCtrl_GMII0_GTXCLK    0x0BF0
#define PINCtrl_GMII0_RXCLK     0x0BF4
#define PINCtrl_GMII0_RXD0      0x0BF8
#define PINCtrl_GMII0_RXD1      0x0BFC
#define PINCtrl_GMII0_RXD2      0x0C00
#define PINCtrl_GMII0_RXD3      0x0C04
#define PINCtrl_GMII0_RXD4      0x0C08
#define PINCtrl_GMII0_RXD5      0x0C0C
#define PINCtrl_GMII0_RXD6      0x0C10
#define PINCtrl_GMII0_RXD7      0x0C14
#define PINCtrl_GMII0_RXDV      0x0C18
#define PINCtrl_GMII0_RXER      0x0C1C
#define PINCtrl_GMII0_TXCLK     0x0C20
#define PINCtrl_GMII0_TXD0      0x0C24
#define PINCtrl_GMII0_TXD1      0x0C28
#define PINCtrl_GMII0_TXD2      0x0C2C
#define PINCtrl_GMII0_TXD3      0x0C30
#define PINCtrl_GMII0_TXD4      0x0C34
#define PINCtrl_GMII0_TXD5      0x0C38
#define PINCtrl_GMII0_TXD6      0x0C3C
#define PINCtrl_GMII0_TXD7      0x0C40
#define PINCtrl_GMII0_TXEN      0x0C44
#define PINCtrl_MDIO_MCLK       0x0C48
#define PINCtrl_MDIO_MDIO       0x0C4C
#define PINCtrl_VLYNQ_CLOCK     0x0C50
#define PINCtrl_VLYNQ_SCRUN     0x0C54
#define PINCtrl_VLYNQ_TXD0      0x0C58
#define PINCtrl_VLYNQ_TXD1      0x0C5C
#define PINCtrl_VLYNQ_TXD2      0x0C60
#define PINCtrl_VLYNQ_TXD3      0x0C64
#define PINCtrl_VLYNQ_RXD0      0x0C68
#define PINCtrl_VLYNQ_RXD1      0x0C6C
#define PINCtrl_VLYNQ_RXD2      0x0C70
#define PINCtrl_VLYNQ_RXD3      0x0C74
#define PINCtrl_IIC0_SCL        0x0C78
#define PINCtrl_IIC0_SDA        0x0C7C
#define PINCtrl_IIC1_SCL        0x0C80
#define PINCtrl_IIC1_SDA        0x0C84
#define PINCtrl_GP0_IO0         0x0C88
#define PINCtrl_GP0_IO1         0x0C8C
#define PINCtrl_GP0_IO2         0x0C90
#define PINCtrl_GP0_IO3         0x0C94
#define PINCtrl_GP0_IO4         0x0C98
#define PINCtrl_GP0_IO5         0x0C9C
#define PINCtrl_GP0_IO6         0x0CA0
#define PINCtrl_GP0_IO7         0x0CA4
#define PINCtrl_GP1_IO30        0x0CA8
#define PINCtrl_GP1_IO31        0x0CAC
#define PINCtrl_HDMI_SCL        0x0CB0
#define PINCtrl_HDMI_SDA        0x0CB4
#define PINCtrl_HDMI_CEC        0x0CB8
#define PINCtrl_HDMI_HPDET      0x0CBC
#define PINCtrl_TCLK            0x0CC0
#define PINCtrl_RTCK            0x0CC4
#define PINCtrl_TDI             0x0CC8
#define PINCtrl_TDO             0x0CCC
#define PINCtrl_TMS             0x0CD0
#define PINCtrl_TRSTN           0x0CD4
#define PINCtrl_EMU0            0x0CD8
#define PINCtrl_EMU1            0x0CDC
#define PINCtrl_EMU2            0x0CE0
#define PINCtrl_EMU3            0x0CE4
#define PINCtrl_EMU4            0x0CE8

#endif

//Jason merged form Dick for SONTIA
#define M0 0
#define M1 1
#define M2 2
#define M3 3
#define IPD (0 << 3)
#define IPU (2 << 3)
#define PD_EN (1 << 3)

#define TI816X_CTRL_BASE				CTRL_BASE

#define DM8168_MUX_VAL(OFFSET,VALUE)\
    __raw_writel((VALUE), TI816X_CTRL_BASE + (OFFSET));

/*
 * To get the physical address the offset has
 * to be added to TI816X_CTRL_BASE
 */

#define PINCTRL1                                  0x0800
#define PINCTRL2                                  0x0804
#define PINCTRL3                                  0x0808
#define PINCTRL4                                  0x080C
#define PINCTRL5                                  0x0810
#define PINCTRL6                                  0x0814
#define PINCTRL7                                  0x0818
#define PINCTRL8                                  0x081C
#define PINCTRL9                                  0x0820
#define PINCTRL10                                 0x0824
#define PINCTRL11                                 0x0828
#define PINCTRL12                                 0x082C
#define PINCTRL13                                 0x0830
#define PINCTRL14                                 0x0834
#define PINCTRL15                                 0x0838
#define PINCTRL16                                 0x083C
#define PINCTRL17                                 0x0840
#define PINCTRL18                                 0x0844
#define PINCTRL19                                 0x0848
#define PINCTRL20                                 0x084C
#define PINCTRL21                                 0x0850
#define PINCTRL22                                 0x0854
#define PINCTRL23                                 0x0858
#define PINCTRL24                                 0x085C
#define PINCTRL25                                 0x0860
#define PINCTRL26                                 0x0864
#define PINCTRL27                                 0x0868
#define PINCTRL28                                 0x086C
#define PINCTRL29                                 0x0870
#define PINCTRL30                                 0x0874
#define PINCTRL31                                 0x0878
#define PINCTRL32                                 0x087C
#define PINCTRL33                                 0x0880
#define PINCTRL34                                 0x0884
#define PINCTRL35                                 0x0888
#define PINCTRL36                                 0x088C
#define PINCTRL37                                 0x0890
#define PINCTRL38                                 0x0894
#define PINCTRL39                                 0x0898
#define PINCTRL40                                 0x089C
#define PINCTRL41                                 0x08A0
#define PINCTRL42                                 0x08A4
#define PINCTRL43                                 0x08A8
#define PINCTRL44                                 0x08AC
#define PINCTRL45                                 0x08B0
#define PINCTRL46                                 0x08B4
#define PINCTRL47                                 0x08B8
#define PINCTRL48                                 0x08BC
#define PINCTRL49                                 0x08C0
#define PINCTRL50                                 0x08C4
#define PINCTRL51                                 0x08C8
#define PINCTRL52                                 0x08CC
#define PINCTRL53                                 0x08D0
#define PINCTRL54                                 0x08D4
#define PINCTRL55                                 0x08D8
#define PINCTRL56                                 0x08DC
#define PINCTRL57                                 0x08E0
#define PINCTRL58                                 0x08E4
#define PINCTRL59                                 0x08E8
#define PINCTRL60                                 0x08EC
#define PINCTRL61                                 0x08F0
#define PINCTRL62                                 0x08F4
#define PINCTRL63                                 0x08F8
#define PINCTRL64                                 0x08FC
#define PINCTRL65                                 0x0900
#define PINCTRL66                                 0x0904
#define PINCTRL67                                 0x0908
#define PINCTRL68                                 0x090C
#define PINCTRL69                                 0x0910
#define PINCTRL70                                 0x0914
#define PINCTRL71                                 0x0918
#define PINCTRL72                                 0x091C
#define PINCTRL73                                 0x0920
#define PINCTRL74                                 0x0924
#define PINCTRL75                                 0x0928
#define PINCTRL76                                 0x092C
#define PINCTRL77                                 0x0930
#define PINCTRL78                                 0x0934
#define PINCTRL79                                 0x0938
#define PINCTRL80                                 0x093C
#define PINCTRL81                                 0x0940
#define PINCTRL82                                 0x0944
#define PINCTRL83                                 0x0948
#define PINCTRL84                                 0x094C
#define PINCTRL85                                 0x0950
#define PINCTRL86                                 0x0954
#define PINCTRL87                                 0x0958
#define PINCTRL88                                 0x095C
#define PINCTRL89                                 0x0960
#define PINCTRL90                                 0x0964
#define PINCTRL91                                 0x0968
#define PINCTRL92                                 0x096C
#define PINCTRL93                                 0x0970
#define PINCTRL94                                 0x0974
#define PINCTRL95                                 0x0978
#define PINCTRL96                                 0x097C
#define PINCTRL97                                 0x0980
#define PINCTRL98                                 0x0984
#define PINCTRL99                                 0x0988
#define PINCTRL100                                0x098C
#define PINCTRL101                                0x0990
#define PINCTRL102                                0x0994
#define PINCTRL103                                0x0998
#define PINCTRL104                                0x099C
#define PINCTRL105                                0x09A0
#define PINCTRL106                                0x09A4
#define PINCTRL107                                0x09A8
#define PINCTRL108                                0x09AC
#define PINCTRL109                                0x09B0
#define PINCTRL110                                0x09B4
#define PINCTRL111                                0x09B8
#define PINCTRL112                                0x09BC
#define PINCTRL113                                0x09C0
#define PINCTRL114                                0x09C4
#define PINCTRL115                                0x09C8
#define PINCTRL116                                0x09CC
#define PINCTRL117                                0x09D0
#define PINCTRL118                                0x09D4
#define PINCTRL119                                0x09D8
#define PINCTRL120                                0x09DC
#define PINCTRL121                                0x09E0
#define PINCTRL122                                0x09E4
#define PINCTRL123                                0x09E8
#define PINCTRL124                                0x09EC
#define PINCTRL125                                0x09F0
#define PINCTRL126                                0x09F4
#define PINCTRL127                                0x09F8
#define PINCTRL128                                0x09FC
#define PINCTRL129                                0x0A00
#define PINCTRL130                                0x0A04
#define PINCTRL131                                0x0A08
#define PINCTRL132                                0x0A0C
#define PINCTRL133                                0x0A10
#define PINCTRL134                                0x0A14
#define PINCTRL135                                0x0A18
#define PINCTRL136                                0x0A1C
#define PINCTRL137                                0x0A20
#define PINCTRL138                                0x0A24
#define PINCTRL139                                0x0A28
#define PINCTRL140                                0x0A2C
#define PINCTRL141                                0x0A30
#define PINCTRL142                                0x0A34
#define PINCTRL143                                0x0A38
#define PINCTRL144                                0x0A3C
#define PINCTRL145                                0x0A40
#define PINCTRL146                                0x0A44
#define PINCTRL147                                0x0A48
#define PINCTRL148                                0x0A4C
#define PINCTRL149                                0x0A50
#define PINCTRL150                                0x0A54
#define PINCTRL151                                0x0A58
#define PINCTRL152                                0x0A5C
#define PINCTRL153                                0x0A60
#define PINCTRL154                                0x0A64
#define PINCTRL155                                0x0A68
#define PINCTRL156                                0x0A6C
#define PINCTRL157                                0x0A70
#define PINCTRL158                                0x0A74
#define PINCTRL159                                0x0A78
#define PINCTRL160                                0x0A7C
#define PINCTRL161                                0x0A80
#define PINCTRL162                                0x0A84
#define PINCTRL163                                0x0A88
#define PINCTRL164                                0x0A8C
#define PINCTRL165                                0x0A90
#define PINCTRL166                                0x0A94
#define PINCTRL167                                0x0A98
#define PINCTRL168                                0x0A9C
#define PINCTRL169                                0x0AA0
#define PINCTRL170                                0x0AA4
#define PINCTRL171                                0x0AA8
#define PINCTRL172                                0x0AAC
#define PINCTRL173                                0x0AB0
#define PINCTRL174                                0x0AB4
#define PINCTRL175                                0x0AB8
#define PINCTRL176                                0x0ABC
#define PINCTRL177                                0x0AC0
#define PINCTRL178                                0x0AC4
#define PINCTRL179                                0x0AC8
#define PINCTRL180                                0x0ACC
#define PINCTRL181                                0x0AD0
#define PINCTRL182                                0x0AD4
#define PINCTRL183                                0x0AD8
#define PINCTRL184                                0x0ADC
#define PINCTRL185                                0x0AE0
#define PINCTRL186                                0x0AE4
#define PINCTRL187                                0x0AE8
#define PINCTRL188                                0x0AEC
#define PINCTRL189                                0x0AF0
#define PINCTRL190                                0x0AF4
#define PINCTRL191                                0x0AF8
#define PINCTRL192                                0x0AFC
#define PINCTRL193                                0x0B00
#define PINCTRL194                                0x0B04
#define PINCTRL195                                0x0B08
#define PINCTRL196                                0x0B0C
#define PINCTRL197                                0x0B10
#define PINCTRL198                                0x0B14
#define PINCTRL199                                0x0B18
#define PINCTRL200                                0x0B1C
#define PINCTRL201                                0x0B20
#define PINCTRL202                                0x0B24
#define PINCTRL203                                0x0B28
#define PINCTRL204                                0x0B2C
#define PINCTRL205                                0x0B30
#define PINCTRL206                                0x0B34
#define PINCTRL207                                0x0B38
#define PINCTRL208                                0x0B3C
#define PINCTRL209                                0x0B40
#define PINCTRL210                                0x0B44
#define PINCTRL211                                0x0B48
#define PINCTRL212                                0x0B4C
#define PINCTRL213                                0x0B50
#define PINCTRL214                                0x0B54
#define PINCTRL215                                0x0B58
#define PINCTRL216                                0x0B5C
#define PINCTRL217                                0x0B60
#define PINCTRL218                                0x0B64
#define PINCTRL219                                0x0B68
#define PINCTRL220                                0x0B6C
#define PINCTRL221                                0x0B70
#define PINCTRL222                                0x0B74
#define PINCTRL223                                0x0B78
#define PINCTRL224                                0x0B7C
#define PINCTRL225                                0x0B80
#define PINCTRL226                                0x0B84
#define PINCTRL227                                0x0B88
#define PINCTRL228                                0x0B8C
#define PINCTRL229                                0x0B90
#define PINCTRL230                                0x0B94
#define PINCTRL231                                0x0B98
#define PINCTRL232                                0x0B9C
#define PINCTRL233                                0x0BA0
#define PINCTRL234                                0x0BA4
#define PINCTRL235                                0x0BA8
#define PINCTRL236                                0x0BAC
#define PINCTRL237                                0x0BB0
#define PINCTRL238                                0x0BB4
#define PINCTRL239                                0x0BB8
#define PINCTRL240                                0x0BBC
#define PINCTRL241                                0x0BC0
#define PINCTRL242                                0x0BC4
#define PINCTRL243                                0x0BC8
#define PINCTRL244                                0x0BCC
#define PINCTRL245                                0x0BD0
#define PINCTRL246                                0x0BD4
#define PINCTRL247                                0x0BD8
#define PINCTRL248                                0x0BDC
#define PINCTRL249                                0x0BE0
#define PINCTRL250                                0x0BE4
#define PINCTRL251                                0x0BE8
#define PINCTRL252                                0x0BEC
#define PINCTRL253                                0x0BF0
#define PINCTRL254                                0x0BF4
#define PINCTRL255                                0x0BF8
#define PINCTRL256                                0x0BFC
#define PINCTRL257                                0x0C00
#define PINCTRL258                                0x0C04
#define PINCTRL259                                0x0C08
#define PINCTRL260                                0x0C0C
#define PINCTRL261                                0x0C10
#define PINCTRL262                                0x0C14
#define PINCTRL263                                0x0C18
#define PINCTRL264                                0x0C1C
#define PINCTRL265                                0x0C20
#define PINCTRL266                                0x0C24
#define PINCTRL267                                0x0C28
#define PINCTRL268                                0x0C2C
#define PINCTRL269                                0x0C30
#define PINCTRL270                                0x0C34
#define PINCTRL271                                0x0C38
#define PINCTRL272                                0x0C3C
#define PINCTRL273                                0x0C40
#define PINCTRL274                                0x0C44
#define PINCTRL275                                0x0C48
#define PINCTRL276                                0x0C4C
#define PINCTRL277                                0x0C50
#define PINCTRL278                                0x0C54
#define PINCTRL279                                0x0C58
#define PINCTRL280                                0x0C5C
#define PINCTRL281                                0x0C60
#define PINCTRL282                                0x0C64
#define PINCTRL283                                0x0C68
#define PINCTRL284                                0x0C6C
#define PINCTRL285                                0x0C70
#define PINCTRL286                                0x0C74
#define PINCTRL287                                0x0C78
#define PINCTRL288                                0x0C7C
#define PINCTRL289                                0x0C80
#define PINCTRL290                                0x0C84
#define PINCTRL291                                0x0C88
#define PINCTRL292                                0x0C8C
#define PINCTRL293                                0x0C90
#define PINCTRL294                                0x0C94
#define PINCTRL295                                0x0C98
#define PINCTRL296                                0x0C9C
#define PINCTRL297                                0x0CA0
#define PINCTRL298                                0x0CA4
#define PINCTRL299                                0x0CA8
#define PINCTRL300                                0x0CAC
#define PINCTRL301                                0x0CB0
#define PINCTRL302                                0x0CB4
#define PINCTRL303                                0x0CB8
#define PINCTRL304                                0x0CBC
#define PINCTRL305                                0x0CC0
#define PINCTRL306                                0x0CC4
#define PINCTRL307                                0x0CC8
#define PINCTRL308                                0x0CCC
#define PINCTRL309                                0x0CD0
#define PINCTRL310                                0x0CD4
#define PINCTRL311                                0x0CD8
#define PINCTRL312                                0x0CDC
#define PINCTRL313                                0x0CE0
#define PINCTRL314                                0x0CE4
#define PINCTRL315                                0x0CE8
#define PINCTRL316                                0x0CEC
#define PINCTRL317                                0x0CF0
#define PINCTRL318                                0x0CF4
#define PINCTRL319                                0x0CF8
#define PINCTRL320                                0x0CFC
#define PINCTRL321                                0x0D00
#define PINCTRL322                                0x0D04
#define PINCTRL323                                0x0D08

#endif /* __TI81XX_HARDWARE_H */

